
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

#include <GL/gl.h>

#include "defines.h"
#include "atofont_core.h"
#include "atofont_s.h"



/* draw the characters one by one */
static int _atofont_string_write(struct atofont *font, unsigned char *u) {

  struct atocharacter *c;
  float texcoords[4*2];
  float fg, m;


  glVertexPointer(2, GL_FLOAT, 0, font->vertices);
  glTexCoordPointer(2, GL_FLOAT, 0, texcoords);

  m = 0.0f;

  if (font->type == ATOFONT_TYPE_FIXED) {
    fg = font->scale_x + font->spacing;
    while (*u != 0) {
      c = &(font->characters[*(u++)]);

      texcoords[0*2 + 0] = c->x;
      texcoords[0*2 + 1] = c->y + font->character_fixed_height;
      texcoords[1*2 + 0] = c->x + font->character_fixed_width;
      texcoords[1*2 + 1] = c->y + font->character_fixed_height;
      texcoords[2*2 + 0] = c->x;
      texcoords[2*2 + 1] = c->y;
      texcoords[3*2 + 0] = c->x + font->character_fixed_width;
      texcoords[3*2 + 1] = c->y;

      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
      glTranslatef(fg, 0, 0);
    }
  }
  else {
    while (*u != 0) {
      c = &(font->characters[*(u++)]);

      texcoords[0*2 + 0] = c->x;
      texcoords[0*2 + 1] = c->y + font->character_fixed_height;
      texcoords[1*2 + 0] = c->x + font->character_fixed_width;
      texcoords[1*2 + 1] = c->y + font->character_fixed_height;
      texcoords[2*2 + 0] = c->x;
      texcoords[2*2 + 1] = c->y;
      texcoords[3*2 + 0] = c->x + font->character_fixed_width;
      texcoords[3*2 + 1] = c->y;

      fg = c->left_margin*font->scale_x;
      glTranslatef(-fg + m, 0, 0);
      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
      m = fg + c->width*font->scale_x + font->spacing;
    }
  }

  return SUCCEEDED;
}


int atofont_string_write_billboard(struct atofont *font, char *s, float x, float y, float z, int flags) {

  float p;


  if (font == NULL || s == NULL)
    return FAILED;

  /* fix the modelview */
  _atofont_billboard_spherical_cheat_apply(x, y, z);

  if ((flags & ATOFONT_CENTERED) != 0) {
    p = _atofont_get_string_width(font, (unsigned char *)s);
    glTranslatef(-p/2.0f, -font->scale_y/2.0f, 0.0f);
  }

  /* write the string */
  _atofont_string_write(font, (unsigned char *)s);

  /* return the modelview */
  glPopMatrix();

  return SUCCEEDED;
}


int atofont_string_write_2D_centered(struct atofont *font, char *s, float x, float y) {

  float p;


  if (font == NULL || s == NULL)
    return FAILED;

  p = _atofont_get_string_width(font, (unsigned char *)s);
  p = -p/2;

  glPushMatrix();
  glTranslatef(p + x, -font->scale_y/2.0f + y, 0);

  /* write the string */
  _atofont_string_write(font, (unsigned char *)s);

  glPopMatrix();

  return SUCCEEDED;
}


int atofont_string_write_2D(struct atofont *font, char *s, float x, float y, int flags) {

  float p;


  if (font == NULL || s == NULL)
    return FAILED;

  glPushMatrix();

  if ((flags & ATOFONT_CENTERED) != 0) {
    p = _atofont_get_string_width(font, (unsigned char *)s);
    glTranslatef(-p/2.0f + x, -font->scale_y/2.0f + y, 0);
  }
  else
    glTranslatef(x, y, 0);

  /* write the string */
  _atofont_string_write(font, (unsigned char *)s);

  glPopMatrix();

  return SUCCEEDED;
}
