
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <time.h>

#include <SDL/SDL.h>
#include <SDL/SDL_video.h>
#include <SDL/SDL_keysym.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "defines.h"
#include "main.h"
#include "sdl.h"
#include "opengl.h"
#include "png.h"
#include "../atofont_core.h"
#include "../atofont_so.h"
#include "../atofont_s.h"


/* quit key status */
int quit = NO;

/* example fonts */
struct atofont *ef_fixed = NULL, *ef_proportional = NULL;
struct atostringobject *so_fixed = NULL, *so_proportional = NULL;

/* texture */
unsigned char *td = NULL;
int tx = 0, ty = 0, tt;



int main(int argc, char *argv[]) {

  srand(time(NULL));

  /* load the font */
  png_load("examplefont.png", &tx, &ty, &tt, &td);

  /* create fonts */
  atofont_create(&ef_fixed, td, tt, tx, ty, 16, 16);
  atofont_create(&ef_proportional, td, tt, tx, ty, 16, 16);
  atofont_set_type(ef_proportional, ATOFONT_TYPE_PROPORTIONAL);

  /* init the fonts */
  atofont_set_scaling(ef_fixed, 0.08, 0.08);
  atofont_set_spacing(ef_fixed, 0.01);
  atofont_set_scaling(ef_proportional, 0.08, 0.08);
  atofont_set_spacing(ef_proportional, 0.01);

  /* create the string objects */
  atofont_string_object_create(ef_fixed, "F: String Object", &so_fixed);
  atofont_string_object_create(ef_proportional, "P: String Object", &so_proportional);

  if (sdl_init() == FAILED)
    return 1;
  if (opengl_init(td, tt, tx, ty) == FAILED)
    return 1;

  /* main loop */
  while (quit == NO) {
    sdl_keyboard_io();
    opengl_render_scene();
  }

  /* free the string objects */
  atofont_string_object_free(so_fixed);
  atofont_string_object_free(so_proportional);

  /* free the fonts */
  atofont_free(ef_fixed);
  atofont_free(ef_proportional);

  opengl_exit();
  sdl_exit();

  return 0;
}
